@@ -1,11 +1,21 @@
 package Net::SFTP::Foreign::Common;
 
-our $VERSION = '1.66_01';
+our $VERSION = '1.68_01';
 
 use strict;
 use warnings;
 use Carp;
-use Scalar::Util qw(dualvar tainted);
+
+BEGIN {
+    # Some versions of Scalar::Util are crippled
+    require Scalar::Util;
+    eval { Scalar::Util->import(qw(dualvar tainted)); 1 }
+        or do {
+            *tainted = sub { croak "The version of Scalar::Util installed on your system "
+                                 . "does not provide 'tainted'" };
+            *dualvar = sub { $_[0] };
+        };
+}
 
 use Net::SFTP::Foreign::Helpers qw(_gen_wanted _ensure_list _debug _glob_to_regex _is_lnk _is_dir $debug);
 use Net::SFTP::Foreign::Constants qw(:status);
@@ -59,7 +69,9 @@ sub _set_error {
 	    $str = $code ? "Unknown error $code" : "OK";
 	}
         $debug and $debug & 64 and _debug("_set_err code: $code, str: $str");
-	return $sftp->{_error} = dualvar $code, $str;
+	my $error = $sftp->{_error} = dualvar $code, $str;
+        croak $error if $sftp->{_autodie};
+        return $error;
     }
     else {
 	return $sftp->{_error} = 0;
@@ -131,6 +143,7 @@ sub find {
     my $dirs = delete $opts{dirs};
     my $follow_links = delete $opts{follow_links};
     my $on_error = delete $opts{on_error};
+    local $self->{_autodie} if $on_error;
     my $realpath = delete $opts{realpath};
     my $ordered = delete $opts{ordered};
     my $names_only = delete $opts{names_only};
@@ -262,6 +275,7 @@ sub glob {
     return () if $glob eq '';
 
     my $on_error = delete $opts{on_error};
+    local $sftp->{_autodie} if $on_error;
     my $follow_links = delete $opts{follow_links};
     my $ignore_case = delete $opts{ignore_case};
     my $names_only = delete $opts{names_only};
